' === Temp9700A ==============================================
' This program measures temperature using a MCP9700A sensor.
' The program is adapted from the one presented by Svejk,
' in "PICAXE-08M2 Temperature Sensing with MCP9700/9700A."

' === Directives ===
  #com 6                     'specify serial port
  #picaxe 20M2               'specify processor
  #terminal 4800             'open the terminal window
  #no_data                   'save time downloading
  
' === Constants ======
  symbol pwr9700 = C.0       'MCP9700A V+ connected to C.0
  symbol  sensor = C.1       'temperature ADC channel

' === Variables =====
  symbol  ADCval = w0        'stores raw temperature data
  symbol   tempC = w1        'stores Celsius temp (x10)
  symbol   tempF = w2        'stores Fahrenheit temp (x10)
  symbol degrees = b6        'stores integer value of temp
  symbol  tenths = b7        'stores tenths value of temp
  symbol   signC = b8        'stores sign of Celsius temp
  symbol   signF = b9        'stores sign of Fahrenheit temp
  
' === Begin Main Program ======================================

dirsB = %11111111            'portB pins are all outputs
dirsC = %10111101            'pin C.0 is ADC input
adcconfig %011               'Vref+ is FVR, Vref- is 0V

do
  high pwr9700               'power up MCP9700A
  pause 10                   '9700 startup delay
  fvrsetup FVR2048           'set FVR at 2048mV
  readadc10 sensor,ADCval    'fetch raw data
  low pwr9700                'turn off MCP9700A
  
  gosub convertRaw2temp
  degrees = tempC / 10
  tenths  = tempC //10
  sertxd (signC,#degrees,".",#tenths,"C = ")
  
  gosub convertC2F
  degrees = tempF / 10
  tenths  = tempF //10
  
  if tenths >= 5 then
    inc degrees
  endif
  
  sertxd (signF,#degrees,"F",CR,LF)
  wait 5                     'slow it down a bit
loop

' === End Main Program: Subroutines Follow ====================
convertRaw2temp:
  if ADCval < 250 then            'test for sign
    tempC = 2 * ADCval
    tempC = 500-tempC
    signC = "-"
  else
    tempC = 2 * ADCval - 500
    signC = "+"
  endif
  return
  
convertC2F:
  select case ADCval
    case 0 to 160                 'C=-50 to -17.7 (F negative)
      tempF = tempC *9 / 5 - 320
      signF = "-"
    case 161 to 249               'C=-17.8 to -0.1 (F positive)           '
      tempF = tempC *9 / 5
      tempF = 320 - tempF
      signF = "+" 
    case 250 to 870               'C= 0 to +125 (F=+32 to +255)
      tempF = tempC *9 / 5 + 320
      signF = "+"
  endselect 
  return